@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright © 2011 – 2017 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,8 +24,8 @@
 #endif
 
 #define __GOA_INSIDE_GOA_H__
-#include <goa/goatypes.h>
 #include <goa/goaclient.h>
+#include <goa/goaenums.h>
 #include <goa/goaerror.h>
 #include <goa/goaversion.h>
 #include <goa/goa-generated.h>
