@@ -248,7 +248,7 @@ ensure_credentials_sync (GoaProvider         *provider,
                                          cancellable,
                                          error))
     {
-      if (error != NULL)
+      if (error != NULL && g_error_matches (*error, GOA_ERROR, GOA_ERROR_NOT_AUTHORIZED))
         {
           g_prefix_error (error,
                           /* Translators: the first %s is the username
@@ -259,8 +259,6 @@ ensure_credentials_sync (GoaProvider         *provider,
                           username,
                           g_quark_to_string ((*error)->domain),
                           (*error)->code);
-          (*error)->domain = GOA_ERROR;
-          (*error)->code = GOA_ERROR_NOT_AUTHORIZED;
         }
       goto out;
     }
@@ -314,6 +312,10 @@ on_username_or_server_changed (GtkEditable    *editable,
   const char *server = NULL;
   const char *username = NULL;
 
+  /* Reset the preference to ignore SSL/TLS errors
+   */
+  data->accept_ssl_errors = FALSE;
+
   email = gtk_editable_get_text (GTK_EDITABLE (data->email_address));
   if (!goa_utils_parse_email_address (email, NULL, NULL))
     goto out;
@@ -338,6 +340,10 @@ on_email_or_password_changed (GtkEditable    *editable,
   g_autofree char *email_localpart = NULL;
   g_autofree char *email_domain = NULL;
 
+  /* Reset the preference to ignore SSL/TLS errors
+   */
+  data->accept_ssl_errors = FALSE;
+
   email = gtk_editable_get_text (GTK_EDITABLE (data->email_address));
   if (!goa_utils_parse_email_address (email, &email_localpart, &email_domain))
     goto out;
@@ -387,24 +393,6 @@ create_setup_page (GoaProvider    *provider,
   data->password = goa_provider_dialog_add_password_entry (dialog, group, _("_Password"));
   goa_provider_dialog_add_description (dialog, NULL, _("Exchange account details will be auto-detected from your email address when possible"));
 
-  if (new_account)
-    {
-      group = goa_provider_dialog_add_group (dialog, _("Account Details"));
-      data->username = goa_provider_dialog_add_entry (dialog, group, _("User_name"));
-      data->server = goa_provider_dialog_add_entry (dialog, group, _("_Domain"));
-      goa_provider_dialog_add_description (dialog, data->server, _("Example domain: example.com"));
-
-      g_signal_connect (data->username,
-                        "changed",
-                        G_CALLBACK (on_username_or_server_changed),
-                        data);
-      g_signal_connect (data->server,
-                        "changed",
-                        G_CALLBACK (on_username_or_server_changed),
-                        data);
-    }
-
-  gtk_widget_grab_focus ((new_account) ? data->email_address : data->password);
   g_signal_connect (data->email_address,
                     "changed",
                     G_CALLBACK (on_email_or_password_changed),
@@ -413,6 +401,22 @@ create_setup_page (GoaProvider    *provider,
                     "changed",
                     G_CALLBACK (on_email_or_password_changed),
                     data);
+
+  group = goa_provider_dialog_add_group (dialog, _("Account Details"));
+  data->username = goa_provider_dialog_add_entry (dialog, group, _("User_name"));
+  data->server = goa_provider_dialog_add_entry (dialog, group, _("_Domain"));
+  goa_provider_dialog_add_description (dialog, data->server, _("Example domain: example.com"));
+
+  g_signal_connect (data->username,
+                    "changed",
+                    G_CALLBACK (on_username_or_server_changed),
+                    data);
+  g_signal_connect (data->server,
+                    "changed",
+                    G_CALLBACK (on_username_or_server_changed),
+                    data);
+
+  gtk_widget_grab_focus ((new_account) ? data->email_address : data->password);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -458,6 +462,7 @@ add_account_autodiscover_cb (GoaEwsClient *client,
 
   if (!goa_ews_client_autodiscover_finish (client, res, &error))
     {
+      data->accept_ssl_errors = (error->code == GOA_ERROR_SSL);
       goa_provider_dialog_report_error (data->dialog, error);
       return;
     }
