@@ -123,108 +123,6 @@ uri_encode_identity (const char *uri_string,
   return NULL;
 }
 
-/*< private >
- * dav_normalize_uri:
- * @base_uri: a URI
- * @uri_ref: (nullable): an absolute or relative URI
- * @server: (nullable) (out): location for server name
- *
- * Normalize a URI to http(s) with a trailing `/`.
- *
- * If @uri_ref is given it will be resolved relative to @base_uri, before
- * the trailing `/` is applied.
- *
- * If @server is not %NULL, it will be set to a presentable name.
- *
- * Returns: (transfer full): a new URI, or %NULL
- */
-static char *
-dav_normalize_uri (const char  *base_uri,
-                   const char  *uri_ref,
-                   char       **server)
-{
-  g_autoptr (GUri) uri = NULL;
-  g_autoptr (GUri) uri_out = NULL;
-  const char *scheme;
-  const char *path;
-  g_autofree char *new_path = NULL;
-  g_autofree char *uri_string = NULL;
-  int std_port = 0;
-
-  /* dav(s) is used by DNS-SD and gvfs */
-  scheme = g_uri_peek_scheme (base_uri);
-  if (scheme == NULL)
-    {
-      uri_string = g_strconcat ("https://", base_uri, NULL);
-      scheme = "https";
-      std_port = 443;
-    }
-  else if (g_str_equal (scheme, "https")
-           || g_str_equal (scheme, "davs"))
-    {
-      uri_string = g_strdup (base_uri);
-      scheme = "https";
-      std_port = 443;
-    }
-  else if (g_str_equal (scheme, "http")
-           || g_str_equal (scheme, "dav"))
-    {
-      uri_string = g_strdup (base_uri);
-      scheme = "http";
-      std_port = 80;
-    }
-  else
-    {
-      g_critical ("Unsupported URI scheme \"%s\"", scheme);
-      return NULL;
-    }
-
-  uri = g_uri_parse (uri_string, G_URI_FLAGS_ENCODED | G_URI_FLAGS_PARSE_RELAXED, NULL);
-  if (uri == NULL)
-    return NULL;
-
-  if (uri_ref != NULL)
-    {
-      uri_out = g_uri_parse_relative (uri, uri_ref, G_URI_FLAGS_ENCODED | G_URI_FLAGS_PARSE_RELAXED, NULL);
-      if (uri_out == NULL)
-        return NULL;
-
-      g_uri_unref (uri);
-      uri = g_steal_pointer (&uri_out);
-    }
-
-  path = g_uri_get_path (uri);
-  if (!g_str_has_suffix (path, "/"))
-    new_path = g_strconcat (path, "/", NULL);
-
-  uri_out = g_uri_build (g_uri_get_flags (uri),
-                         scheme,
-                         g_uri_get_userinfo (uri),
-                         g_uri_get_host (uri),
-                         g_uri_get_port (uri),
-                         new_path ? new_path : path,
-                         g_uri_get_query (uri),
-                         g_uri_get_fragment (uri));
-
-  if (server != NULL)
-    {
-      g_autofree char *port_string = NULL;
-      g_autofree char *pretty_path = NULL;
-      int port;
-
-      port = g_uri_get_port (uri_out);
-      port_string = g_strdup_printf (":%d", port);
-
-      path = g_uri_get_path (uri_out);
-      pretty_path = g_strdup (path);
-      pretty_path[strlen(pretty_path) - 1] = '\0';
-
-      *server = g_strconcat (g_uri_get_host (uri), (port == std_port || port == -1) ? "" : port_string, pretty_path, NULL);
-    }
-
-  return g_uri_to_string (uri_out);
-}
-
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gboolean on_handle_get_password (GoaPasswordBased      *interface,
@@ -339,9 +237,9 @@ static struct
   const char *service;
   const char *key;
 } keyfile_endpoints[] = {
+  { GOA_SERVICE_TYPE_WEBDAV, "Uri" },
   { GOA_SERVICE_TYPE_CALDAV, "CalDavUri" },
   { GOA_SERVICE_TYPE_CARDDAV, "CardDavUri" },
-  { GOA_SERVICE_TYPE_WEBDAV, "Uri" },
 };
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -379,7 +277,7 @@ ensure_credentials_sync (GoaProvider         *provider,
       g_autofree char *uri = NULL;
 
       uri = goa_util_lookup_keyfile_string (object, keyfile_endpoints[i].key);
-      if (uri != NULL && *uri != '\0')
+      if (uri != NULL && g_uri_is_valid (uri, SOUP_HTTP_URI_FLAGS, NULL))
         {
           g_autoptr(GoaDavConfig) config = NULL;
 
@@ -394,16 +292,9 @@ ensure_credentials_sync (GoaProvider         *provider,
         }
     }
 
-  if (!ret && error != NULL)
+  if (!ret)
     {
-      if (*error == NULL)
-        {
-          g_set_error_literal (error,
-                               GOA_ERROR,
-                               GOA_ERROR_FAILED,
-                               _("Unknown error"));
-        }
-      else
+      if (error != NULL && g_error_matches (*error, GOA_ERROR, GOA_ERROR_NOT_AUTHORIZED))
         {
           g_prefix_error (error,
                           /* Translators: the first %s is the username
@@ -414,8 +305,6 @@ ensure_credentials_sync (GoaProvider         *provider,
                           username,
                           g_quark_to_string ((*error)->domain),
                           (*error)->code);
-          (*error)->domain = GOA_ERROR;
-          (*error)->code = GOA_ERROR_NOT_AUTHORIZED;
         }
 
       return ret;
@@ -477,8 +366,12 @@ on_uri_username_or_password_changed (GtkEditable *editable, gpointer user_data)
   const char *address;
   g_autofree char *uri = NULL;
 
+  /* Reset the preference to ignore SSL/TLS errors
+   */
+  data->accept_ssl_errors = FALSE;
+
   address = gtk_editable_get_text (GTK_EDITABLE (data->uri));
-  uri = dav_normalize_uri (address, NULL, NULL);
+  uri = goa_utils_normalize_url (address, NULL, NULL);
 
   if (uri != NULL)
     {
@@ -541,7 +434,7 @@ create_account_details_ui (GoaProvider    *provider,
       for (size_t i = 0; i < G_N_ELEMENTS (keyfile_endpoints); i++)
         {
           uri = goa_util_lookup_keyfile_string (data->object, keyfile_endpoints[i].key);
-          if (uri != NULL && *uri != '\0')
+          if (uri != NULL && g_uri_is_valid (uri, SOUP_HTTP_URI_FLAGS, NULL))
             {
               data->check_config = goa_dav_config_new (keyfile_endpoints[i].service, uri, username);
               break;
@@ -657,6 +550,7 @@ add_account_handle_response (GTask     *task,
                              GPtrArray *services)
 {
   AddAccountData *data = g_task_get_task_data (task);
+  GtkEditable *editable = NULL;
   const char *username = NULL;
   const char *base_uri = NULL;
   const char *check_uri = NULL;
@@ -686,7 +580,8 @@ add_account_handle_response (GTask     *task,
 
           data->check_stage = GOA_PROVIDER_FEATURE_FILES;
           service = GOA_SERVICE_TYPE_WEBDAV;
-          check_uri = gtk_editable_get_text (GTK_EDITABLE (data->webdav_uri));
+          editable = GTK_EDITABLE (data->webdav_uri);
+          check_uri = gtk_editable_get_text (editable);
           break;
 
         /* WebDAV: discover so we get the redirected URI for GVfs, but don't
@@ -701,7 +596,8 @@ add_account_handle_response (GTask     *task,
 
           data->check_stage = GOA_PROVIDER_FEATURE_CALENDAR;
           service = GOA_SERVICE_TYPE_CALDAV;
-          check_uri = gtk_editable_get_text (GTK_EDITABLE (data->caldav_uri));
+          editable = GTK_EDITABLE (data->caldav_uri);
+          check_uri = gtk_editable_get_text (editable);
           break;
 
         /* CalDAV/CardDAV: user-defined URIs override discovered services even
@@ -716,7 +612,8 @@ add_account_handle_response (GTask     *task,
 
           data->check_stage = GOA_PROVIDER_FEATURE_CONTACTS;
           service = GOA_SERVICE_TYPE_CARDDAV;
-          check_uri = gtk_editable_get_text (GTK_EDITABLE (data->carddav_uri));
+          editable = GTK_EDITABLE (data->carddav_uri);
+          check_uri = gtk_editable_get_text (editable);
           break;
 
         case GOA_PROVIDER_FEATURE_CONTACTS:
@@ -728,6 +625,7 @@ add_account_handle_response (GTask     *task,
 
           /* Set the next stage to default to add_account_credentials() */
           data->check_stage = GOA_PROVIDER_FEATURE_TICKETING;
+          editable = NULL;
           break;
 
         default:
@@ -741,7 +639,7 @@ add_account_handle_response (GTask     *task,
     }
 
   /* If the user entered a bunk URI, they probably want to be notified */
-  normalized_uri = dav_normalize_uri (base_uri, check_uri, NULL);
+  normalized_uri = goa_utils_normalize_url (base_uri, check_uri, NULL);
   if (normalized_uri == NULL)
     {
       g_set_error (&error,
@@ -752,6 +650,12 @@ add_account_handle_response (GTask     *task,
       goa_provider_dialog_report_error (data->dialog, error);
       return FALSE;
     }
+  else if (editable != NULL)
+    {
+      g_signal_handlers_block_by_func (editable, on_uri_username_or_password_changed, data);
+      gtk_editable_set_text (editable, normalized_uri);
+      g_signal_handlers_unblock_by_func (editable, on_uri_username_or_password_changed, data);
+    }
 
   data->check_config = goa_dav_config_new (service, normalized_uri, username);
 
@@ -770,6 +674,7 @@ add_account_check_cb (GoaDavClient *client,
 
   if (!goa_dav_client_check_finish (client, result, &error))
     {
+      data->accept_ssl_errors = (error->code == GOA_ERROR_SSL);
       goa_provider_dialog_report_error (data->dialog, error);
       return;
     }
@@ -804,6 +709,7 @@ add_account_discover_cb (GoaDavClient *client,
   services = goa_dav_client_discover_finish (client, result, &error);
   if (services == NULL)
     {
+      data->accept_ssl_errors = (error->code == GOA_ERROR_SSL);
       goa_provider_dialog_report_error (data->dialog, error);
       return;
     }
@@ -862,7 +768,7 @@ add_account_action_cb (GoaProviderDialog *dialog,
   password = gtk_editable_get_text (GTK_EDITABLE (data->password));
 
   g_free (data->presentation_identity);
-  uri = dav_normalize_uri (uri_text, NULL, &server);
+  uri = goa_utils_normalize_url (uri_text, NULL, &server);
   if (strchr (username, '@') != NULL)
     data->presentation_identity = g_strdup (username);
   else
@@ -881,6 +787,12 @@ add_account_action_cb (GoaProviderDialog *dialog,
       return;
     }
 
+  /* Update the entry field with the URL we're actually using.
+   */
+  g_signal_handlers_block_by_func (data->uri, on_uri_username_or_password_changed, data);
+  gtk_editable_set_text (GTK_EDITABLE (data->uri), uri);
+  g_signal_handlers_unblock_by_func (data->uri, on_uri_username_or_password_changed, data);
+
   /* Confirm the account */
   dav_client = goa_dav_client_new ();
   goa_dav_client_discover (dav_client,
@@ -932,10 +844,12 @@ refresh_account_credentials_cb (GoaAccount   *account,
                                 gpointer      user_data)
 {
   g_autoptr(GTask) task = G_TASK (g_steal_pointer (&user_data));
+  AddAccountData *data = g_task_get_task_data (task);
   GError *error = NULL;
 
   if (!goa_account_call_ensure_credentials_finish (account, NULL, res, &error))
     {
+      data->accept_ssl_errors = (error->code == GOA_ERROR_SSL);
       goa_provider_task_return_error (task, error);
       return;
     }
@@ -1162,3 +1076,4 @@ on_handle_get_password (GoaPasswordBased      *interface,
 
   return TRUE; /* invocation was handled */
 }
+
